#!/usr/bin/env python

"""
This script is used for collecting data from MT4 history center.

To use this script you must download the data of the required
currency pair from MT4 history center first, this guild shows
the steps ti do this: 
https://www.metatrader4.com/en/trading-platform/help/service/history_center

Usage:
      fx_data_collector [options]

Options:
      -h, --help            : show this help message and exit.
      
      -v, --version         : show the version of FX-Manager and exit.

      -rdt, --raw_data_type : Supported Raw Data types:
                                    - daily_candles  : 1-min candles (OHLCV) in daily seperate files.
                                      (a file for each currency_pair for each day)
                                    - period_candles : 1-min candles (OHLCV) over a period.
                                      (a single file for each currency_pair)
      
      -cps, --currency_pairs: list of symbols of the desired currency pairs of which data is collected.
                              symbolS are separated by a white blanc character (space).

      -s, --start           : start date of the period of which data is collected. must be passed in the
                              form YYYY_mm_dd.

      -e, --end             : end date of the period of which data is collected. must be passed in the
                              form YYYY_mm_dd.
      
      -st, save_to          : path to the directory where data is saved, could be absolut or relative. 
"""

doc = __doc__

from os import getcwd
import argparse
import pandas as pd
from fxmanager.dwx.rates_historic import rates_historic as rh
import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)

# Add Optional arguments
parser = argparse.ArgumentParser(add_help=False)
parser.add_argument('-h','--help', default=False, action='store_true')
parser.add_argument('-v','--version', action='version', version=__version__)
parser.add_argument('-rdt','--raw_data_type', type=str, default = 'daily_candles')
parser.add_argument('-cps','--currency_pairs', type=str, nargs='+', default = ['EURUSD'])
parser.add_argument('-s','--start', type=str, default = '2021_09_20')
parser.add_argument('-e','--end', type=str, default = '2021_09_21')
parser.add_argument('-st','--save_to', type=str, default = getcwd())

# Parse arguments
in_args = parser.parse_args()

if in_args.help:
    import sys
    print(doc)
    sys.exit()

cps = [x.upper() for x in in_args.currency_pairs]

start_date = pd.to_datetime(in_args.start, format='%Y_%m_%d')
end_date = pd.to_datetime(in_args.end, format='%Y_%m_%d')
dates = pd.date_range(start=start_date, end=end_date)

# Create a data collector object
dc = rh()

for cp in cps:
    if in_args.raw_data_type == 'daily_candles':
        for date in dates:
            # remove weekends
            if date.weekday() == 5 or date.weekday() == 6:
                continue
            dc.get_daily_candles(save_to = in_args.save_to, currency_pair=cp, date='.'.join(str(date.date()).split('-')))

    elif in_args.raw_data_type == 'period_candles':
        # remove weekends
        dates = dates[dates.weekday != 5] & dates[dates.weekday != 6]
        dc.get_period_candles(save_to = in_args.save_to, currency_pair=cp, start='.'.join(str(dates[0].date()).split('-')), end='.'.join(str(dates[-1].date()).split('-')))


#!/usr/bin/env python

"""
This script runs a portfolio optimization test using historical data.

Assuming the dataset has N working days sorted in folders (0, 1, 2, ...)
in 'data\\raw_data\\' directory, this test uses the data of each single
day (n) to construct an optimized portfolio of a compination of currency
pairs, then uses the constructed portfolio to calculate the expected 
returns if this portfolio is used for trading in the next day (n+1).

a portfolio consists of one or more assets where an asset in this context
refers to a currency pair being traded in a specefic time frame. for example:
if the currency pair 'EURUSD' is being traded in two different time frames,
5 and 10 minutes (a new position is opened every 5 or 10 minutes), in this case,
the 'EURUSD_5min' and 'EURUSD_10min' are two different assets.

Usage:
      run_fx_opt_test [options]

Options:
      -h, --help                     : show this help message and exit.
      
      -v, --version                  : show the version of FX-Manager and exit.
      
      -om, --optimization_method     : the supported optimization methods are:
                                        - mixed_timeframes: selects the optimal currency pair
                                                            from each available timeframe, then
                                                            selects the optimal equally weighted
                                                            combination of these time frames.
                                        - single_timeframe: selects the optimal time frame from
                                                            the available timeframes, then selects
                                                            the optimal equally weighted combination
                                                            of currency pairs in the selected time frame.

      -obj, --optimization_objective : the supported optimization objectives are:
                                        - min_win_rate   : minimizes the required win rate for
                                                           the portfolio to acheive a positive
                                                           returns.
                                        - min_risk_reward: minimizes the required risk reward
                                                           ratio required for the portfolio to
                                                           acheive a positive returns.

      -spd, --scenarios_per_day      : number of scenarios over which the expected return
                                       is averaged every day (must be integer)

      -rr, --risk_rewards            : list of risk-reward ratios for which the expected
                                       return is calculated. (must be floats in quote marks
                                       and separated by spaces)
      
      -lvrg, --leverage              : leverage used for calculating expected returns 
                                       (must be float in quote marks)
      
      -eqw, --equally_weighted       : boolean flag, if True, the assets in the constucted
                                       portfolios will be equally weighted.

      -sap, --single_asset_portfolio : boolean flag, if True, the constucted portfolios will
                                       consist of a single asset.

      -sp, --save_plots              : boolean flag, if True, the program will save result 
                                       plots to 'data\\visualizations\\' directory.
                                       
      -sl, --save_logs               : boolean flag, if True, the program logs are saved to
                                       'data\\logs\\equally_weighted_optimizer_logs.txt' file.

===========================================================================================
===========================================================================================

Saved Results: 
    ** test results are saved to 'data\\visualizations' for plots and
       'data\\stats' for csv files.
        
    - Optimizer_Test_Win_Probs_&_Expexted_Rets_vs_Offsets.png :
        image with 2 sub-plots sharing the X-axis for the offsets, and Y axis for win 
        probabilites and average expected returns. Offsets are linearly separated
        floats added to the 'min_win_rate' argument, starting from (0.01, 0.02, ...),
        the value of 'min_win_rate' is incremented until the win probabilty becomes
        greater than or equal to 1. the plot might also contain several lines per plot,
        each line represents calculations a single risk-reward ratio.
    
    - Optimizer_Test_Required_Win_Rates.png : 
        image with a plot of constructed portfolios win rates on the Y-axis vs days on the X-axis.
    
    - Optimizer_Test_Risk _Reward_Ratios.png : 
        image with a plot of constructed portfolios risk rewards on the Y-axis vs days on the X-axis.

    - Optimizer_Test_portfolios.csv : 
        CSV file with the constructed portfolios for every day. the index starts from 1 because every
        row represents the portfolio created using data from the day before so day 0 does not count.

    - Optimizer_Test_win_probs.csv :
        CSV file with the data frame used for plotting the first sub-plot in
        'Optimizer_Test_Win_Probs_&_Expexted_Rets_vs_Offsets.png'
    
    - Optimizer_Test_expexted_returns.csv :
        CSV file with the data frame used for plotting the second sub-plot in
        'Optimizer_Test_Win_Probs_&_Expexted_Rets_vs_Offsets.png'
"""

doc = __doc__

import argparse
import sys
if '--equally_weighted' in sys.argv or '-eqw' in sys.argv:
    import fxmanager.optimization.eqw_optimization_test as optim
else:
    import fxmanager.optimization.w_optimization_test as optim

import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)

# Add Optional arguments
parser = argparse.ArgumentParser(add_help=False)
parser.add_argument('-h','--help', default=False, action='store_true')
parser.add_argument('-v','--version', action='version', version=__version__)
parser.add_argument('-om','--optimization_method', type=str, default = 'mixed_timeframes')
parser.add_argument('-obj','--optimization_objective', type=str, default = 'min_win_rate')
parser.add_argument('-spd','--scenarios_per_day', type=int, default = 100)
parser.add_argument('-rr','--risk_rewards', type=float, nargs='+', default = [0.8, 0.9, 1, 1.1, 1.2])
parser.add_argument('-lvrg','--leverage', type=float, default = 1.0)
parser.add_argument('-eqw','--equally_weighted', default=False, action='store_true')
parser.add_argument('-sap','--single_asset_portfolio', default=False, action='store_true')
parser.add_argument('-sp','--save_plots', default=False, action='store_true')
parser.add_argument('-sl','--save_logs', default=False, action='store_true')

# Parse arguments
in_args = parser.parse_args()

if in_args.help:
    import sys
    print(doc)
    sys.exit()
    
## Run the test
optim.run_test(optimization_method = in_args.optimization_method,
                optimization_objective = in_args.optimization_objective,
                scenarios_per_day = in_args.scenarios_per_day,
                risk_rewards = in_args.risk_rewards,
                leverage = in_args.leverage,
                test_run = in_args.single_asset_portfolio,
                save_plots = in_args.save_plots,
                save_logs = in_args.save_logs)


# -*- coding: utf-8 -*-
"""
    DWX_ZMQ_Execution.py
    --
    @author: Darwinex Labs (www.darwinex.com)
    
    Copyright (c) 2019 onwards, Darwinex. All rights reserved.
    
    Licensed under the BSD 3-Clause License, you may not use this file except 
    in compliance with the License. 
    
    You may obtain a copy of the License at:    
    https://opensource.org/licenses/BSD-3-Clause
"""

from pandas import to_datetime
from time import sleep

class DWX_ZMQ_Execution():
    
    def __init__(self, _zmq):
        self._zmq = _zmq
    
    ##########################################################################
    
    def _execute_(self, 
                  _exec_dict,
                  _verbose=False, 
                  _delay=0.1,
                  _wbreak=10):
        
        _check = ''
        
        # Reset thread data output
        self._zmq._set_response_(None)
        
        # OPEN TRADE
        if _exec_dict['_action'] == 'OPEN':
            
            _check = '_action'
            self._zmq._DWX_MTX_NEW_TRADE_(_order=_exec_dict)
            
        # CLOSE TRADE
        elif _exec_dict['_action'] == 'CLOSE':
            
            _check = '_response_value'
            self._zmq._DWX_MTX_CLOSE_TRADE_BY_TICKET_(_exec_dict['_ticket'])
            
        if _verbose:
            print('\n>> DWX MESSAGE >> [{}] {} -> MetaTrader'.format(_exec_dict['_comment'],
                                                   str(_exec_dict)))
            
        # While loop start time reference            
        _ws = to_datetime('now')
        
        # While data not received, sleep until timeout
        while self._zmq._valid_response_('zmq') == False:
            sleep(_delay)
            
            if (to_datetime('now') - _ws).total_seconds() > (_delay * _wbreak):
                break
        
        # If data received, return DataFrame
        if self._zmq._valid_response_('zmq'):
            
            if _check in self._zmq._get_response_().keys():
                return self._zmq._get_response_()
                
        # Default
        return None
    
    ##########################################################################
    


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

try:
    setup(name = 'FX-Manager',
        version = '1.0.6',
        author = 'Abdullah Bahi',
        author_email = 'abdullahbahi@icloud.com',
        description = 'Python package for developing and testing forex algorithmic trading strategies.',
        long_description = long_description,
        long_description_content_type = "text/markdown",
        url = 'https://github.com/AbdullahBahi/fxmanager',
        classifiers = ['Development Status :: 3 - Alpha',
                        'Intended Audience :: Developers',
                        'Topic :: Software Development :: Build Tools',
                        'License :: OSI Approved :: GNU General Public License (GPL)',
                        'Programming Language :: Python :: 3',
                        'Operating System :: OS Independent'],
        keywords = ['forex', 'forex trading', 'algorethmic trading', 'backtesting'],
        scripts = ['scripts\\create_fx_proj.py',
                        'scripts\\fx_data_collector.py', 
                        'scripts\\fx_pp.py',
                        'scripts\\run_fx_hist_sim.py', 
                        'scripts\\run_fx_live_sim.py', 
                        'scripts\\run_fx_opt_test.py'],
        package_dir = {'':'src'},
        packages = find_packages(where='src'),
        install_requires = ['zmq',
                        'pandas>=0.25.1',
                        'numpy>=1.16.5',
                        'matplotlib>=3.1.1'],
        python_requires = '>=3.7'
    )
except:
    print('Error!')



#!/usr/bin/env python

"""
This module contains helper functions used internally in this module and a public function to run live simulation with live price feed from MT4.

Public Functions:
    - run(): starts trading simulation with live prices from MT4 EA.
"""

import sys
from os import listdir, getcwd
from os.path import join
from time import sleep, time, asctime, localtime
import pandas as pd 
from fxmanager.basic.util import preprocess, get_avg_rets
from fxmanager.dwx.prices_subscriptions import prices_subscriptions as ps
import fxmanager.optimization.eq_weight_optimizer as optim
import fxmanager.optimization.weight_optimizer as optim_w
import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)

##########################################################################################################################
##                                                 Helper Functions
##########################################################################################################################

def optimize_portfolio(data_dir, raw_data_format, optimization_method, optimization_objective, weight_optimization=False, is_preprocessed=True):
    """
    helper function to 'fxmanager.simulation.live.run()' function. Does portfolio optimization using data from previous day. The constructed portfolio is used for live simulation.

    Returns:
        - portfolio     : dictionary with keys ('currency_pairs', 'time_frames', 'weights', 'optimized_n', 'optimized_W_R', 'optimized_R_R')
        - currency_pairs: list with portfolio currency pairs.
        - time_frames   : list with portfolio timeframes.
        - weights       : list with portfolio weights.
    """

    # Extract the time frames and currency pairs lists
    time_frames = listdir(join(data_dir, 'time_frames', '0'))
    time_frames.append(time_frames[0])
    time_frames = time_frames[1:]
    currency_pairs = [x[:6] for x in listdir(join(data_dir, 'time_frames', '0', '1min'))]
    
    ## STAGE 1: Preprocessing Data of Previous 24 Hours
    if not is_preprocessed:
        print('\n>> SYSTEM MESSAGE >> STAGE 1: Preprocessing Data of Previous 24 Hours ..\n')
        preprocess(data_dir=data_dir,
                    raw_data_format=raw_data_format,
                    is_live_data=True,
                    save_logs=False)
        print('>> SYSTEM MESSAGE >> Data Processed Successfully!\n')
    else:
        print('\n>> SYSTEM MESSAGE >> STAGE 1: Data of Previous 24 Hours Is Already Preprocessed! Skipping Stage 1 ..\n')
    
    ## STAGE 2: Optimizing Portfolio Parameters
    print('>> SYSTEM MESSAGE >> STAGE 2: Optimizing Portfolio Parameters ..\n')
    # First load best and worst returns
    best_rets_df, wrst_rets_df = get_avg_rets(day=0, data_dir=join(data_dir,'live_data'), time_frames=time_frames, currency_pairs=currency_pairs)
    best_rets_df.index, wrst_rets_df.index = range(best_rets_df.shape[0]), range(best_rets_df.shape[0]) 

    # Now start optimizing the portfolio
    if weight_optimization:
        optimized_portfolio_idxs, optimized_portfolio_tfs, optimized_portfolio_weights, \
        optimized_n, optimized_W_R, optimized_R_R = optim_w.optimize(best_rets_df,
                                                                    wrst_rets_df,
                                                                    method=optimization_method,
                                                                    objective=optimization_objective,
                                                                    test_run=True)
    else:
        optimized_portfolio_idxs, optimized_portfolio_tfs, optimized_portfolio_weights, \
        optimized_n, optimized_W_R, optimized_R_R = optim.optimize(best_rets_df,
                                                                    wrst_rets_df,
                                                                    method=optimization_method,
                                                                    objective=optimization_objective,
                                                                    test_run=True)
    optimized_portfolio_cps = [currency_pairs[x] for x in optimized_portfolio_idxs]
    
    portfolio = {'currency_pairs':optimized_portfolio_cps,
                'time_frames':optimized_portfolio_tfs,
                'weights':optimized_portfolio_weights,
                'optimized_n':optimized_n, 'optimized_W_R':optimized_W_R, 'optimized_R_R':optimized_R_R}
    
    currency_pairs = portfolio['currency_pairs']
    time_frames = portfolio['time_frames']
    weights = portfolio['weights']

    print('>> SYSTEM MESSAGE >> Portfilio Construction is complete! ..\n')

    return portfolio, currency_pairs, time_frames, weights

def get_price(currency_pairs, price_feed, sleep_time):
    """
    helper function to 'fxmanager.simulation.live.run()' function. gets the current prices of portfolio assets from price feed.

    Returns:
        - price_every_iter_df: pandas dataframe with length = 1 and (4* number of currency pairs) columns. Each asset has 4 columns as follows, (000000_bid_open, 000000_ask_open, 000000_bid_close, 000000_ask_close) where 000000 is replaced with currency pair symbol.
    """

    t = time()
    price_every_iter = {}
    
    # get open prices
    for cp in currency_pairs:
        prices = price_feed._recent_prices[cp]
        price_every_iter[cp+'_bid_open'] = min(prices)
        price_every_iter[cp+'_ask_open'] = max(prices)

    # get close prices
    while (time()-t) < (sleep_time*60):
        for cp in currency_pairs:
            prices = price_feed._recent_prices[cp]
            price_every_iter[cp+'_bid_close'] = min(prices)
            price_every_iter[cp+'_ask_close'] = max(prices)
    
    price_every_iter_df = pd.DataFrame(price_every_iter, index=[0])
    return price_every_iter_df

def print_live_state(account, win_rate):
    """
    helper function to 'fxmanager.simulation.live.run()' function. prints the current state of the portfolio.
    """

    print('-----------------------------------------------------------------------------------------------------------')
    print(f'\n>> PROCESS MESSAGE >>  Account State At {asctime(localtime(time()))}:')
    print(f'>> PROCESS MESSAGE >> Balance          : {round(account._balance,2)}$')
    print(f'>> PROCESS MESSAGE >> Equity           : {round(account._live_equity,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin           : {round(account._margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Free Margin      : {round(account._live_free_margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin Level     : {round(account._live_margin_level*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Win Rate         : {round(win_rate*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Open Positions   : {len(account._positions)}')
    for ticket in account._positions:
        order_type = account._positions[ticket]['order_type']
        open_price = account._positions[ticket]['open_price']
        cp = account._positions[ticket]['base_currency'] + account._positions[ticket]['quote_currency']
        print(f'                                       : [{order_type} {cp}] At Open Price: {round(open_price,5)}')
    print(f'>> PROCESS MESSAGE >> Live Profit      : {round(account._live_profit,2)}$\n')
    print(f'>> PROCESS MESSAGE >> Total Profit     : {round(account._profit,2)}$\n')

def print_final_state(account, win_rate):
    """
    helper function to 'fxmanager.simulation.live.run()' function. prints the final state of the portfolio after the simulation is finished.
    """

    print('>> PROCESS MESSAGE >> Printing Final Portfolio State ..\n')
    print(f'>> PROCESS MESSAGE >> Balance          : {round(account._balance,2)}$')
    print(f'>> PROCESS MESSAGE >> Equity           : {round(account._equity,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin           : {round(account._margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Free Margin      : {round(account._free_margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin Level     : {round(account._margin_level*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Open Positions   : {len(account._positions)}')
    print(f'>> PROCESS MESSAGE >> Win Rate         : {round(win_rate*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Total Profit     : {round(account._profit,2)}$\n')

def close_position(account, ticket, portfolio_prices, portfolio_orders, wins, losses):
    """
    helper function to 'fxmanager.simulation.live.run()' function. closes an opened position by ticket.

    Returns:
        - portfolio_orders: pandas dataframe with history of closed positions. Each row represents a closed position.
        - wins            : integer indicating number of closed positions with positive profit.
        - losses          : integer indicating number of closed positions with nigative profit.
        - win_rate        : float indicating percentage of wins to (wins+losses).
    """

    cp = account._positions[ticket]['base_currency'] + account._positions[ticket]['quote_currency']
    open_price = account._positions[ticket]['open_price']
    volume = account._positions[ticket]['volume']
    tf = account._positions[ticket]['time_frame']
    order_type = account._positions[ticket]['order_type']
    SL = account._positions[ticket]['SL']
    TP = account._positions[ticket]['TP']
    w = account._positions[ticket]['weight']

    close_price = portfolio_prices.iloc[-1][[cp+'_ask_close', cp+'_bid_close']]
    close_price.index = ['ask', 'bid']
    position_porfit, margin_req, close_price = account.close_position(ticket=ticket, prices=close_price)
    
    order = pd.DataFrame({'Ticket': ticket,
                        'Order Type':order_type,
                        'Currency Pair':cp,
                        'Time Frame': tf,
                        'Weight': w,
                        'Volume': volume,
                        'SL':SL,
                        'TP':TP,
                        'Open Price': open_price,
                        'Close Price': close_price,
                        'Margin requirment': margin_req,
                        'Porfit': position_porfit}, index=[0])
    
    portfolio_orders = pd.concat([portfolio_orders, order], ignore_index=True)
    
    if position_porfit >= 0:
        wins += 1
    else:
        losses +=1
    
    win_rate = wins / (wins+losses)

    return portfolio_orders, wins, losses, win_rate

##########################################################################################################################
##                                                 Main Function
##########################################################################################################################

def run(account, strategy, data_dir=None, construct_portfolio=False, portfolio={}, weight_optimization=False, is_preprocessed=True,raw_data_format='',
        optimization_method='', optimization_objective='', sleep_time=1, risk_factor=1, sync_zero=False, save_logs=False, **kwargs):
    """
    starts trading simulation with live prices from MT4 EA.

    Args: 
        - account               : account object with all account information.
        - strategy              : strategy object with the trading strategy information.
        - data_dir              : string with the directory or full path to the directory in which application data is kept. If the setup() function is used to create the recommended project structure, the default None value should be used.
        - construct_portfolio   : boolean indicating whether portfolio optimization is used or not.
        - portfolio             : dictionary with the portfolio to be used for trading, keys are: (currency_pairs, time_frames, weights). This argument is only used if 'construct_portfolio' is set to False.
        - weight_optimization   : boolean indicating whether weight optimization is used in portfolio optimization. This argument is only used when 'construct_portfolio' is set to True.
        - is_preprocessed       : boolean indicating if the data of used for portfolio optimization is preprocessed or not. This argument is only used when 'construct_portfolio' is set to True.
        - raw_data_format       : dictionary with information about raw data used for portfolio optimization. This argument is only used when 'construct_portfolio' is set to True.
        - optimization_method   : string with optimization method to be used. This argument is only used when 'construct_portfolio' is set to True.
        - optimization_objective: string with optimization objective to be used. This argument is only used when 'construct_portfolio' is set to True.
        - sleep_time            : integer indicating the time every which the portfolio is updated (in seconds).
        - risk_factor           : a float with range from 0 to 1 indicating the percentage of reinvested balance.
        - sync_zero             : boolean flag, if True, the simulation starts when the seconds in current time = 0.
        - save_logs             : boolean flag, if the program logs are saved to 'data_dir\\logs\\live_simulation_logs.txt' file.
        - kwargs                : dictionary to hold any number of arguments required for the strategy object.
    Returns:
        - None
    """

    if data_dir is None:
        data_dir = join(getcwd(), 'data')
    if save_logs:
        sys.stdout = open(join(data_dir,'logs','back_test_logs.txt'), 'w')

    print('-----------------------------------------------------------------------------------------------------------')
    print('-----------------------------------------------------------------------------------------------------------')
    print(f'                                   >> {asctime(localtime(time()))} <<')
    print('-----------------------------------------------------------------------------------------------------------')
    print('-----------------------------------------------------------------------------------------------------------')

    if construct_portfolio:
        portfolio, currency_pairs, time_frames, weights = optimize_portfolio(data_dir=data_dir,
                                                                              raw_data_format=raw_data_format,
                                                                              optimization_method=optimization_method,
                                                                              optimization_objective=optimization_objective,
                                                                              weight_optimization=weight_optimization,
                                                                              is_preprocessed = is_preprocessed)
    else:
        print('\n>> SYSTEM MESSAGE >> Portfolio Construction Is Not Needed! Skipping Stages 1 & 2 ..\n')
        currency_pairs = portfolio['currency_pairs']
        time_frames = portfolio['time_frames']
        weights = portfolio['weights']


    # STAGE 3: Subscribe to The Price Feed of Currency Pairs in The Portfolio
    print('>> SYSTEM MESSAGE >> STAGE 3: Subscribing to The Price Feed of Currency Pairs in The Portfolio ..\n')
    price_feed = ps(_symbols=currency_pairs)
    price_feed.run()
    print('\n>> SYSTEM MESSAGE >> Successfully Connected To Live Price Feed!\n')

    ## STAGE 4: The Main Loop
    print('>> SYSTEM MESSAGE >> STAGE 4: Starting The Main Loop ..\n')
    
    # initialize variables
    portfolio_prices = pd.DataFrame()
    portfolio_orders = pd.DataFrame()
    wins = 0
    losses = 0
    win_rate = 0
    try:
        # wait until initial prices are available
        for cp in currency_pairs:
            while price_feed._recent_prices[cp] == 0:
                pass
        
        # wait until seconds == 0 befor starting the main loop (to sync with local time)
        if sync_zero:
            while localtime(time()).tm_sec != 0:
                pass
        
        ## Main Loop
        while not price_feed.isFinished():
            try:
                price_every_iter_df = get_price(currency_pairs=currency_pairs, price_feed=price_feed, sleep_time=sleep_time)
                portfolio_prices = pd.concat([portfolio_prices, price_every_iter_df], ignore_index=True)

                # Loop Through the opened positions and close the ones with period = 0
                tickets = list(account._positions.keys())
                for ticket in tickets:
                    period = account._positions[ticket]['period']
                    if period == 0:
                        portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                                ticket=ticket,
                                                                                portfolio_prices=portfolio_prices,
                                                                                portfolio_orders=portfolio_orders,
                                                                                wins=wins,
                                                                                losses=losses)
                
                # Loop Through portfolio assets and add positions
                for cp, tf, w in zip(currency_pairs, time_frames, weights):
                    if tf[1].isalpha():
                        period = int(int(tf[0]) / sleep_time)
                    else:
                        period = int(int(tf[:2]) / sleep_time)
                    
                    if (portfolio_prices.index[-1] % period) == 0:
                        strategy._currency_pair = cp
                        strategy._time_frame = tf
                        orders = strategy.get_orders(prices = portfolio_prices, **kwargs)
                        order = orders.iloc[-1]
                        order_idx = orders.index[-1]
                        order_ticket = cp + '_' + tf + str(order_idx)
                        if order['order_type'] == 'hold':
                            continue
                            
                        # if the position is already opened, don't open it again
                        if order_ticket not in account._positions:
                            open_price = portfolio_prices.iloc[order_idx][[cp+'_ask_open', cp+'_bid_open']]
                            open_price.index = ['ask', 'bid']
                            is_opened = account.open_pisition(ticket = order_ticket,
                                                                base_currency = cp[:3],
                                                                quote_currency = cp[3:],
                                                                time_frame=tf,
                                                                weight = w,
                                                                SL = order['SL'],
                                                                TP = order['TP'],
                                                                order_type = order['order_type'],
                                                                prices = open_price,
                                                                period = period,
                                                                order_idx = order_idx,
                                                                risk_factor = risk_factor)
                            # if the position couldn't be opened, break out of the main loop and print account state
                            if not is_opened:
                                print('\n>> PROCESS MESSAGE >> Balance is not enough to open a new position!\n')
                                price_feed.stop()
                                print('\n>> PROCESS MESSAGE >> Closing any open positions ..\n')
                                tickets = list(account._positions.keys())
                                for ticket in tickets:
                                    portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                                            ticket=ticket,
                                                                                            portfolio_prices=portfolio_prices,
                                                                                            portfolio_orders=portfolio_orders,
                                                                                            wins=wins,
                                                                                            losses=losses)
                                break
                if not is_opened:
                    break

                # Update account state with current prices
                account.update(prices = portfolio_prices)

                # print account state every minute
                print_live_state(account=account, win_rate=win_rate)
                
            except UnboundLocalError:
                pass

            # if 24 hours (1440 minutes) have passed, finish the program
            if len(portfolio_prices) >= 1440:
                print('\n-----------------------------------------------------------------------------------------------------------')
                print('>> PROCESS MESSAGE >> Congratulations! you have made it through the day. Closing any remaining positions ..\n')
                price_feed.stop()
                tickets = list(account._positions.keys())
                for ticket in tickets:
                    portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                            ticket=ticket,
                                                                            portfolio_prices=portfolio_prices,
                                                                            portfolio_orders=portfolio_orders,
                                                                            wins=wins,
                                                                            losses=losses)
                break
            else:
                print('>> PROCESS MESSAGE >> Waiting for next update! to stop and save the results press CTRL+C\n')

    except KeyboardInterrupt:
        price_feed.stop()
        print('\n>> PROCESS MESSAGE >> Process is terminated by user. Closing any remaining positions ..\n')
        tickets = list(account._positions.keys())
        for ticket in tickets:
            portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                    ticket=ticket,
                                                                    portfolio_prices=portfolio_prices,
                                                                    portfolio_orders=portfolio_orders,
                                                                    wins=wins,
                                                                    losses=losses)
    
    print('>> SYSTEM MESSAGE >> All Positions Are Closed Successfully!\n')
    print('>> SYSTEM MESSAGE >> STAGE 5: Saving Stats & Visualizations ..\n')

    # Print final state of the account
    print_final_state(account=account, win_rate=win_rate)

    # Save the optimized portfolio
    portfolio_df = pd.DataFrame(portfolio)
    portfolio_df.to_csv(join(data_dir,'stats', 'live_porfolio.csv'))

    # Save the orders
    portfolio_orders.to_csv(join(data_dir,'stats', 'live_orders.csv'))

    print('\n>> SYSTEM MESSAGE >> Execution Finished')

    if save_logs:
        sys.stdout.close()
    return



"""
This sub-package contains a modified version of Darwinex ZeroMQ connector which is used for connecting client code to MT4 trading platform.

Public Modules:
    - prices_subscription : This is a modified version of an example of using the Darwinex ZeroMQ Connector for Python 3 and MT4 PULL REQUEST.
    - rates_historic      : This Module is used as an API to get historic data from the MT4 EA.
"""

from . import prices_subscriptions
from . import rates_historic
import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)


#!/usr/bin/env python

"""
This module contains simple implementation of momentum trading strategy.

Public Functions:
    - get_orders(): sample trading strategy for purposes of testing live and historic trading simulators.
"""

import pandas as pd 
import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)

def get_orders(prices, **kwargs):
    """
    sample trading strategy for purposes of testing live and historic trading simulators.

    Args:
        - prices: pandas DataFrame with range index and a the following columns:
            - ask_open: open ask price of the candle stick.
            - bid_open: open bid price of the candle stick.
            - ask_close: close ask price of the candle stick.
            - bid_close: close bid price of the candle stick.
        - kwargs: special dictionary to hold any number of required arguments for the function.
    
    Returns:
        - orders: pandas DataFrame with ONE RAW and the following columns:
            - order_type: can be one of 3 values (buy, sell, hold)
            - SL : stop loss.
            - TP : take profit.
    """

    look_back = kwargs['look_back']

    diff = prices['bid_close'] - prices['bid_open'] 
    
    if len(diff) < look_back:
        look_back = len(diff)

    diff_sum = diff.rolling(look_back+1).sum()
    up_trend_stregnth = diff_sum.loc[(diff_sum >= 0)]  
    down_trend_stregnth = diff_sum.loc[(diff_sum < 0)]

    # normalize stregnth
    up_trend_stregnth /= up_trend_stregnth.max()
    down_trend_stregnth /= down_trend_stregnth.min()

    orders = pd.DataFrame(index=diff_sum.index)
    
    orders.loc[(diff_sum >= 0), 'order_type'] = 'buy' 
    orders.loc[(diff_sum >= 0), 'SL'] = up_trend_stregnth
    orders.loc[(diff_sum >= 0), 'TP'] = up_trend_stregnth
    

    orders.loc[(diff_sum < 0), 'order_type'] = 'sell'
    orders.loc[(diff_sum < 0), 'SL'] = down_trend_stregnth
    orders.loc[(diff_sum < 0), 'TP'] = down_trend_stregnth

    orders.loc[diff_sum.isna(), 'order_type'] = ['hold']*(look_back) ## stupid positions 
    orders.loc[diff_sum.isna(), 'SL'] = [0]*(look_back) ## stupid positions 
    orders.loc[diff_sum.isna(), 'TP'] = [0]*(look_back) ## stupid positions 
    return orders

if __name__ == '__main__':
    # test case for get_orders_live
    prices = pd.DataFrame({'EURUSD_ask_open':[1.8,1.81,1.82,1,1,1,1,1,1,1],
                           'EURUSD_bid_open':[1.79,1.8,1.81,1,1,1,1,1,1,1],
                           'EURUSD_ask_close':[1.9,1.91,1.92,1,1,1,1,1,1,1],
                           'EURUSD_bid_close':[1.89,1.9,1.91,1,1,1,1,1,1,1],
                           'EURUSD_ask_open':[1.8,1.81,1.82,1,1,1,1,1,1,1],
                           'EURUSD_bid_open':[1.79,1.8,1.81,1,1,1,1,1,1,1],
                           'EURUSD_ask_close':[1.9,1.91,1.92,1,1,1,1,1,1,1],
                           'EURUSD_bid_close':[1.89,1.9,1.91,1,1,1,1,1,1,1]})
    orders = get_orders_live(prices, currency_pair='EURUSD', time_frame='10min', look_back=5)
    print(orders)


#!/usr/bin/env python

"""
This module contains helper functions used internally in this module and a public function to run historical simulation.

Public Functions:
    - run(): starts trading simulation with historic prices.
"""

import sys
from os import getcwd
from os.path import join
import pandas as pd 
import fxmanager._metadata as md
from __main__ import __dict__

__dict__.update(md.__dict__)

##########################################################################################################################
##                                                 Helper Functions
##########################################################################################################################

def get_prices(day, data_dir=None, time_frame='', currency_pair=''):
    """
    helper function to 'fxmanager.simulation.historic.run()' function. gets the bid/ask prices of the given currency_pair & time_frame in the selected day from the 'data_dir\\time_frames' directory.

    Args:
        - day           : integer indicating day number - starts from 0 and ends with the total number of work days in the dataset (num_days-1).
        - data_dir      : string with the directory or full path to the directory in which application data is kept. If the setup() function is used to create the recommended project structure, the default None value should be used.
        - time_frame    : a string of the timeframe from wich the prices is read.
        - currency_pair : a string of the currency pair from wich the prices is read. 
    Returns:
        - df: pandas dataframe of the bid & ask prices with columns (ask_open, bid_open, ask_close, bid_close, tick_volume) and a datetime index from the begining to the end off the day with frequency equal to time_frame.
    """

    if data_dir is None:
        data_dir = join(getcwd(), 'data')
    df = pd.read_csv(join(data_dir, 'time_frames', str(day), time_frame , currency_pair+'_'+time_frame+'.csv'), parse_dates=True)
    df.index = pd.to_datetime(df['time'])
    df.drop(['time', 'best_rets', 'wrst_rets', 'ask_sample1', 'bid_sample1', 'ask_sample2', 'bid_sample2'], axis = 1, inplace = True)
    return df

def print_final_state(account, win_rate):
    """
    helper function to 'fxmanager.simulation.historic.run()' function. prints the final state of the portfolio after the simulation is finished.
    """

    print('>> PROCESS MESSAGE >> Printing Final Portfolio State ..\n')
    print(f'>> PROCESS MESSAGE >> Balance          : {round(account._balance,2)}$')
    print(f'>> PROCESS MESSAGE >> Equity           : {round(account._equity,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin           : {round(account._margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Free Margin      : {round(account._free_margin,2)}$')
    print(f'>> PROCESS MESSAGE >> Margin Level     : {round(account._margin_level*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Open Positions   : {len(account._positions)}')
    print(f'>> PROCESS MESSAGE >> Win Rate         : {round(win_rate*100,2)}%')
    print(f'>> PROCESS MESSAGE >> Total Profit     : {round(account._profit,2)}$\n')

def close_position(account, ticket, portfolio_prices, portfolio_orders, wins, losses):
    """
    helper function to 'fxmanager.simulation.historic.run()' function. closes an opened position by ticket.

    Returns:
        - portfolio_orders: pandas dataframe with history of closed positions. Each row represents a closed position.
        - wins            : integer indicating number of closed positions with positive profit.
        - losses          : integer indicating number of closed positions with nigative profit.
        - win_rate        : float indicating percentage of wins to (wins+losses).
    """

    cp = account._positions[ticket]['base_currency'] + account._positions[ticket]['quote_currency']
    open_price = account._positions[ticket]['open_price']
    volume = account._positions[ticket]['volume']
    tf = account._positions[ticket]['time_frame']
    order_type = account._positions[ticket]['order_type']
    SL = account._positions[ticket]['SL']
    TP = account._positions[ticket]['TP']
    w = account._positions[ticket]['weight']

    close_price = portfolio_prices.iloc[-1][[cp+'_ask_close', cp+'_bid_close']]
    close_price.index = ['ask', 'bid']
    position_porfit, margin_req, close_price = account.close_position(ticket=ticket, prices=close_price)
    
    order = pd.DataFrame({'Ticket': ticket,
                        'Order Type':order_type,
                        'Currency Pair':cp,
                        'Time Frame': tf,
                        'Weight': w,
                        'Volume': volume,
                        'SL':SL,
                        'TP':TP,
                        'Open Price': open_price,
                        'Close Price': close_price,
                        'Margin requirment': margin_req,
                        'Porfit': position_porfit}, index=[0])
    
    portfolio_orders = pd.concat([portfolio_orders, order], ignore_index=True)
    
    if position_porfit >= 0:
        wins += 1
    else:
        losses +=1
    
    win_rate = wins / (wins+losses)

    return portfolio_orders, wins, losses, win_rate

##########################################################################################################################
##                                                 Main Function
##########################################################################################################################

def run(account, strategy, data_dir=None, portfolios={}, risk_factor=0.95, save_logs=False, **kwargs):
    """
    starts trading simulation with historic prices.

    Args: 
        - account               : account object with all account information.
        - strategy              : strategy object with the trading strategy information.
        - data_dir              : string with the directory or full path to the directory in which application data is kept. If the setup() function is used to create the recommended project structure, the default None value should be used.
        - portfolios            : pandas dataframe with columns (currency_pairs, time_frames, weights) and range index of length = num_days
        - risk_factor           : a float with range from 0 to 1 indicating the percentage of reinvested balance.
        - save_logs             : boolean flag, if the program logs are saved to 'data_dir\\logs\\live_simulation_logs.txt' file.
        - kwargs                : dictionary to hold any number of arguments required for the strategy object.
    Returns:
        - None
    """

    if data_dir is None:
        data_dir = join(getcwd(), 'data')
    if save_logs:
        sys.stdout = open(join(data_dir,'logs','historic_simulation_logs.txt'), 'w')

    win_rate = 0
    wins = 0
    losses = 0
    
    for day in portfolios.index:
        print('-----------------------------------------------------------------------------------------------------------')
        print('-----------------------------------------------------------------------------------------------------------')
        print('                                             >> Processing Day {} <<'.format(day))
        print('-----------------------------------------------------------------------------------------------------------')
        print('-----------------------------------------------------------------------------------------------------------')

        ## STAGE 1: Extracting Portfolio Parameters
        print('\n>> SYSTEM MESSAGE >> STAGE 1: Extracting Portfolio Parameters ..\n')
        currency_pairs = portfolios.loc[day, 'currency_pairs']
        time_frames = portfolios.loc[day, 'time_frames']
        weights = portfolios.loc[day, 'weights']
        
        ## STAGE 2 : Get The Prices And Orders Of Each Asset in The Portfolio
        print('>> SYSTEM MESSAGE >> Portfolio Parameters Are Loaded & Extracted Successfully!\n')
        print('>> SYSTEM MESSAGE >> STAGE 2: Getting Prices Of Selected Assets In The Portfolio ..\n')
        
        # Initialize empty dataframes used as placeholders for orders data over the day
        portfolio_orders = pd.DataFrame()
        
        # Initialize a dataframe to hold price updates every minute
        _ = get_prices(day=day, data_dir=data_dir, time_frame=time_frames[0], currency_pair=currency_pairs[0])
        idx = pd.date_range(_.index[0], freq='min', periods=1440)
        cols = []
        for cp in currency_pairs:
            cols.append(cp+'_bid_open')
            cols.append(cp+'_ask_open')
            cols.append(cp+'_bid_close')
            cols.append(cp+'_ask_close')
        portfolio_prices = pd.DataFrame([[0]*len(currency_pairs)*4]*1440, index=idx, columns=cols)
        for cp, tf, w in zip(currency_pairs, time_frames, weights):
            prices = get_prices(day=day, data_dir=data_dir, time_frame=tf, currency_pair=cp)
            for idx in prices.index:
                portfolio_prices.loc[[idx], [cp+'_bid_open', cp+'_ask_open', cp+'_bid_close', cp+'_ask_close']] = \
                prices.loc[[idx], ['bid_open', 'ask_open', 'bid_close', 'ask_close']].values
            
        portfolio_prices = portfolio_prices.replace(to_replace=0, method='ffill')
        portfolio_prices.index = range(len(portfolio_prices))
        portfolio_prices.index.name = 'Minutes'
        ## STAGE3: Start Placing Orders
        print('>> SYSTEM MESSAGE >> Prices Over The Day Are Extracted Successfully! ..\n')
        print('>> SYSTEM MESSAGE >> STAGE 3: Starting To Open Positions ..\n')

        # Main Loop
        for idx in portfolio_prices.index:
            try:
                # Loop Through the opened positions and close the ones with period = 0
                tickets = list(account._positions.keys())
                for ticket in tickets:
                    period = account._positions[ticket]['period']
                    if period == 0:
                        portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                                ticket=ticket,
                                                                                portfolio_prices=portfolio_prices.loc[:idx],
                                                                                portfolio_orders=portfolio_orders,
                                                                                wins=wins,
                                                                                losses=losses)
                # Loop Through portfolio assets and add positions
                for cp, tf, w in zip(currency_pairs, time_frames, weights):
                    if tf[1].isalpha():
                        period = int(int(tf[0]))
                    else:
                        period = int(int(tf[:2]))

                    if (idx % period) == 0:
                        strategy._currency_pair = cp
                        strategy._time_frame = tf
                        orders = strategy.get_orders(prices= portfolio_prices.loc[:idx], **kwargs)
                        order = orders.iloc[-1]
                        order_idx = orders.index[-1]
                        order_ticket = cp + '_' + tf + str(order_idx)
                        if order['order_type'] == 'hold':
                            continue

                        # if the position is already opened, don't open it again
                        if order_ticket not in account._positions:
                            open_price = portfolio_prices.iloc[order_idx][[cp+'_ask_open', cp+'_bid_open']]
                            open_price.index = ['ask', 'bid']
                            is_opened = account.open_pisition(ticket = order_ticket,
                                                                base_currency = cp[:3],
                                                                quote_currency = cp[3:],
                                                                time_frame=tf,
                                                                weight = w,
                                                                SL = order['SL'],
                                                                TP = order['TP'],
                                                                order_type = order['order_type'],
                                                                prices = open_price,
                                                                period = period,
                                                                order_idx = order_idx,
                                                                risk_factor = risk_factor)
                            # if the position couldn't be opened, break out of the main loop and print account state
                            if not is_opened:
                                print('\n>> PROCESS MESSAGE >> Balance is not enough to open a new position!\n')
                                print('\n>> PROCESS MESSAGE >> Closing any open positions ..\n')
                                tickets = list(account._positions.keys())
                                for ticket in tickets:
                                    portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                                            ticket=ticket,
                                                                                            portfolio_prices=portfolio_prices.loc[:idx],
                                                                                            portfolio_orders=portfolio_orders,
                                                                                            wins=wins,
                                                                                            losses=losses)
                                print('>> SYSTEM MESSAGE >> All Positions Are Closed Successfully!\n')

                                # Print final state of the account
                                print_final_state(account=account, win_rate=win_rate)
                                break
                if not is_opened:
                    break
                # Update account state with current prices
                account.update(prices = portfolio_prices.loc[:idx])
            except UnboundLocalError:
                pass
        if not is_opened:
            break

        print('>> PROCESS MESSAGE >> Congratulations! you have made it through the day! Closing any open positions ..\n')
        tickets = list(account._positions.keys())
        for ticket in tickets:
            portfolio_orders, wins, losses, win_rate = close_position(account=account,
                                                                        ticket=ticket,
                                                                        portfolio_prices=portfolio_prices,
                                                                        portfolio_orders=portfolio_orders,
                                                                        wins=wins,
                                                                        losses=losses)
        # Save the orders
        print('>> SYSTEM MESSAGE >> STAGE 4: Saving Stats & Visualizations ..\n')
        portfolio_orders.to_csv(join(data_dir,'stats', 'historical_simulation_orders', 'Day_'+str(day)+'_orders.csv'))
        
        # Print final state of the account
        print_final_state(account=account, win_rate=win_rate)
        
    print('>> SYSTEM MESSAGE >> No More Days are Left! Go get More Data!\n')
    print('>> SYSTEM MESSAGE >> Execution Finished')

    if save_logs:
        sys.stdout.close()
    return 


#!/usr/bin/env python

"""
This script is used for running a historical trading simulation with historical prices.

Usage:
      run_fx_hist_sim [options]

Options:
        -h, --help                   : show this help message and exit.

        -v, --version                : show the version of FX-Manager and exit.

        -blc, --balance              : initial balance to start simulation with.

        -act, --account_type         : detemines lot size, supported account types are:
                                        - standard: lot size = 100,000
                                        - mini    : lot size = 10,000
                                        - micro   : lot size = 1000
                                        - nano    : lot size = 100

        -acc, --account_currency     : currency of initial balance.

        -lvrg, --leverage            : the leverage provided by the broker (expressed as percentage).

        -vbs, --volume_bounds        : minimum and maximum volume per position, passed as 2 floats 
                                       separated by a space.

        -rf, risk_factor             : percentage of the amount of reinvested cash.

        -uds, --user_defined_strategy: boolean flag, if True, simulation is done using user-defined
                                       trading strategy which is defined in 'strategy.py' file.

        -tap, --take_all_prices      : boolean flag that controls the frequency of the range index of
                                       'prices' DataFrame which is passed to strategy function.
                                        - If True:
                                       all the prices history since the begining of the day is passed
                                       to the strategy function, the index is a normal range index 
                                       (0, 1, 2, ...). the time difference between every reading is 
                                       determined by the 'sleep_time' argument in the function:
                                       'fxmanager.simulation.historic.run()'.
                                        - If False:
                                       the frequncy of the prices becomes dependant on the time frame
                                       in which we want to generate an order, for examble, if we want
                                       to open a position in 'EURUSD' in the '5min' time frame, the
                                       index will become (0, 5, 10, ..).

        -nd, --num_days              : total number of work days in the dataset.

        -ops, --optimized_portfolios : boolean flag, if True, the simulation uses the optimized portfolios
                                       saved in 'data\\stats\\Optimizer_test_results.csv'. this option
                                       should only be used after running 'optimizer_test.py' script on
                                       the same historical data used for simulation.

        -usp, --use_single_porfolio  : boolean flag, used if 'optimized_portfolios' is False, set to True 
                                       if you want to use the same portfolio for all the days, if False,
                                       'portfolios' dictionary is used instead.

        -sl, --save_logs             : boolean flag, if True, the program logs are saved to
                                       'data\\logs\\preprocessing_logs.txt' file.

===========================================================================================
===========================================================================================

Saved Results: 
        
        - portfolio_orders.csv: CSV file with all the information of all the orders placed 
                                during the simulation. saved in 'data\\stats\\' directory.
"""

doc = __doc__

import argparse
import sys
from os import getcwd
sys.path.append(getcwd())
from fxmanager.basic.account import Account
from fxmanager.basic.util import get_portfolios
from fxmanager.strategies.template import strategy_template
from fxmanager.strategies.naieve_momentum import get_orders
import fxmanager.simulation.historic as sim
import fxmanager._metadata as md
import strategy as srtg
from __main__ import __dict__

__dict__.update(md.__dict__)

# Add Optional arguments
parser = argparse.ArgumentParser(add_help=False)
parser.add_argument('-h','--help', default=False, action='store_true')
parser.add_argument('-v','--version', action='version', version=__version__)
parser.add_argument('-blc','--balance', type=float, default = 1000.0)
parser.add_argument('-act','--account_type', type=str, default = 'standard')
parser.add_argument('-acc','--account_currency', type=str, default = 'usd')
parser.add_argument('-lvrg','--leverage', type=float, default = 1.0)
parser.add_argument('-vbs','--volume_bounds', type=float, nargs='+', default = [0.01, 8.0])
parser.add_argument('-rf','--risk_factor', type=float, default = 0.98)
parser.add_argument('-uds','--user_defined_strategy', default=False, action='store_true')
parser.add_argument('-tap','--take_all_prices', default=False, action='store_true')
parser.add_argument('-nd','--num_days', type=int, default = 2)
parser.add_argument('-ops','--optimized_portfolios', default=False, action='store_true')
parser.add_argument('-usp','--use_single_porfolio', default=False, action='store_true')
parser.add_argument('-sl','--save_logs', default=False, action='store_true')

# Parse arguments
in_args = parser.parse_args()

if in_args.help:
    import sys
    print(doc)
    sys.exit()

# Create an account object
acc = Account(balance = in_args.balance,
              account_type = in_args.account_type,
              account_currency = in_args.account_currency,
              leverage = in_args.leverage,
              volume_bounds = in_args.volume_bounds)

# Create a strategy object
if in_args.user_defined_strategy:
        strategy = strategy_template(srtg.template, take_all_prices=in_args.take_all_prices)
        kwargs = srtg.kwargs_template()
else:
        strategy = strategy_template(get_orders, take_all_prices=in_args.take_all_prices)
        kwargs = {'look_back':3}

# Get Portfolios
portfolios = get_portfolios(num_days=in_args.num_days,
                            single_portfolio=srtg.get_portfolios_template()[0],
                            portfolios=srtg.get_portfolios_template(),
                            optimized_portfolios=in_args.optimized_portfolios,
                            use_single_porfolio=in_args.use_single_porfolio)

sim.run(account = acc,
        strategy = strategy,
        portfolios = portfolios,
        risk_factor = in_args.risk_factor,
        save_logs = in_args.save_logs,
        **kwargs)




#!/usr/bin/env python

"""
This script is used for running a live trading simulation with real market prices from MT4 platform.

Usage:
      run_fx_live_sim [options]

Options:
        -h, --help                   : show this help message and exit.

        -v, --version                : show the version of FX-Manager and exit.

        -lrdt, live_raw_data_type    : type of raw data used for portfolio optimization.
                                       Supported Raw Data types: 
                                        - daily_bid_ask  : Bid/Ask ticks in daily seperate files.
                                        (a file for each currency_pair in each day)
                                        - daily_candles  : 1-min candles (OHLCV) in daily seperate files.
                                        (a file for each currency_pair in each day)
                                        - period_candles : 1-min candles (OHLCV) over a period.
                                        (a file for each currency_pair for all the days)

        -om, --optimization_method   : the supported optimization methods are:
                                        - mixed_timeframes: selects the optimal currency pair
                                                            from each available timeframe, then
                                                            selects the optimal equally weighted
                                                            combination of these time frames.
                                        - single_timeframe: selects the optimal time frame from
                                                            the available timeframes, then selects
                                                            the optimal equally weighted combination
                                                            of currency pairs in the selected time frame.

        -obj, --optimization_objective : the supported optimization objectives are:
                                          - min_win_rate   : minimizes the required win rate for
                                                             the portfolio to acheive a positive
                                                             returns.
                                          - min_risk_reward: minimizes the required risk reward
                                                             ratio required for the portfolio to
                                                             acheive a positive returns.

        -eqw, --equally_weighted     : boolean flag, if True, the assets in the constucted
                                       portfolios will be equally weighted.

        -blc, --balance              : initial balance to start simulation with.

        -act, --account_type         : detemines lot size, supported account types are:
                                        - standard: lot size = 100,000
                                        - mini    : lot size = 10,000
                                        - micro   : lot size = 1000
                                        - nano    : lot size = 100

        -acc, --account_currency     : currency of initial balance.

        -lvrg, --leverage            : the leverage provided by the broker (expressed as percentage).

        -vbs, --volume_bounds        : minimum and maximum volume per position, passed as 2 floats 
                                       separated by a space.

        

        -uds, --user_defined_strategy: boolean flag, if True, simulation is done using user-defined
                                       trading strategy which is defined in 'strategy.py' file.

        -tap, --take_all_prices      : boolean flag that controls the frequency of the range index of
                                       'prices' DataFrame which is passed to strategy function.
                                        - If True:
                                       all the prices history since the begining of the day is passed
                                       to the strategy function, the index is a normal range index 
                                       (0, 1, 2, ...). the time difference between every reading is 
                                       determined by the 'sleep_time' argument in the function:
                                       'fxmanager.simulation.historic.run()'.
                                        - If False:
                                       the frequncy of the prices becomes dependant on the time frame
                                       in which we want to generate an order, for examble, if we want
                                       to open a position in 'EURUSD' in the '5min' time frame, the
                                       index will become (0, 5, 10, ..).

        -cp, --construct_portfolio   : boolean flag, if True, the data in  of the previous day stored
                                       in 'data\\live_raw_data' is used to construct a portfolio to be
                                       used for simulation

        -st, --sleep_time            : time between each price update (float).

        -rf, risk_factor             : percentage of the amount of reinvested cash (float).

        -sz, --sync_zero             : boolean flag, if True, the simulation starts when the seconds
                                       in current time = 0.

        -sl, --save_logs             : boolean flag, if True, the program logs are saved to
                                       'data\\logs\\preprocessing_logs.txt' file.

===========================================================================================
===========================================================================================

Saved Results: 

        - live_porfolio.csv: CSV file for the portfolio used for simulation. saved in
          'data\\stats\\' directory..
        
        - portfolio_orders.csv: CSV file with all the information of all the orders placed 
                                during the simulation. saved in 'data\\stats\\' directory.
"""

doc = __doc__

import argparse
import sys
from os import getcwd
sys.path.append(getcwd())
import json
from fxmanager.basic.account import Account
from fxmanager.basic.util import get_portfolios
from fxmanager.strategies.template import strategy_template
from fxmanager.strategies.naieve_momentum import get_orders
import fxmanager.simulation.live as sim
import fxmanager._metadata as md
import strategy as srtg
from __main__ import __dict__

__dict__.update(md.__dict__)

# Add Optional arguments
parser = argparse.ArgumentParser(add_help=False)
parser.add_argument('-h','--help', default=False, action='store_true')
parser.add_argument('-v','--version', action='version', version=__version__)

parser.add_argument('-lrdt','--live_raw_data_type', type=str, default = 'daily_candles')
parser.add_argument('-om','--optimization_method', type=str, default = 'mixed_timeframes')
parser.add_argument('-obj','--optimization_objective', type=str, default = 'min_win_rate')
parser.add_argument('-eqw','--equally_weighted', default=False, action='store_true')

parser.add_argument('-blc','--balance', type=float, default = 1000.0)
parser.add_argument('-act','--account_type', type=str, default = 'standard')
parser.add_argument('-acc','--account_currency', type=str, default = 'usd')
parser.add_argument('-lvrg','--leverage', type=float, default = 1.0)
parser.add_argument('-vbs','--volume_bounds', type=float, nargs='+', default = [0.01, 8.0])

parser.add_argument('-uds','--user_defined_strategy', default=False, action='store_true')
parser.add_argument('-tap','--take_all_prices', default=False, action='store_true')

parser.add_argument('-cp','--construct_portfolio', default=False, action='store_true')
parser.add_argument('-st','--sleep_time', type=float, default = 1.0)

parser.add_argument('-rf','--risk_factor', type=float, default = 0.98)
parser.add_argument('-sz','--sync_zero', default=False, action='store_true')
parser.add_argument('-sl','--save_logs', default=False, action='store_true')

# Parse arguments
in_args = parser.parse_args()

if in_args.help:
    import sys
    print(doc)
    sys.exit()

# Preprocessing Information (used only if optimization is used [construct_portfolio==True])
with open('raw_data_formats.json') as f:
    raw_data_formats = json.load(f)
if in_args.live_raw_data_type == 'daily_bid_ask':
    raw_data_format = raw_data_formats['daily_bid_ask']
elif in_args.live_raw_data_type == 'daily_candles':
    raw_data_format = raw_data_formats['daily_candles']
elif in_args.live_raw_data_type == 'period_candles':
    raw_data_format = raw_data_formats['period_candles']
else:
    pass 

# Optimization Information (used only if optimization is used [construct_portfolio==True])
optimization_method = in_args.optimization_method
optimization_objective = in_args.optimization_objective
weight_optimization = not in_args.equally_weighted

# Portfolio Information (used only if optimization is NOT used [construct_portfolio==False])
portfolio = srtg.get_portfolios_template()[0]

# Create an account object
acc = Account(balance = in_args.balance,
            account_type = in_args.account_type,
            account_currency = in_args.account_currency,
            leverage = in_args.leverage,
            volume_bounds = in_args.volume_bounds)

# Create a strategy object
if in_args.user_defined_strategy:
        strategy = strategy_template(srtg.template, take_all_prices=in_args.take_all_prices)
        kwargs = srtg.kwargs_template()
else:
        strategy = strategy_template(get_orders, take_all_prices=in_args.take_all_prices)
        kwargs = {'look_back':3}

# Run Simulation
sim.run(account=acc,
        strategy=strategy,
        construct_portfolio=in_args.construct_portfolio,
        portfolio=portfolio,
        weight_optimization=weight_optimization,
        is_preprocessed=in_args.construct_portfolio,
        raw_data_format=raw_data_format,
        optimization_method=in_args.optimization_method,
        optimization_objective=in_args.optimization_objective,
        sleep_time=in_args.sleep_time, ## In Minutes
        risk_factor=in_args.risk_factor, ## percentage of money to be invested from account balance during simulation
        sync_zero=in_args.sync_zero,
        save_logs=in_args.save_logs,
        **kwargs) ## Multiple of the timeframe (i.e. if an order to be placed in 10 min tf, look back would be 30 minutes with check every 10min)